==> maven-model-builder-3.0.1-sources.jar: org/apache/maven/model/path/DefaultUrlNormalizer.java <==
package org.apache.maven.model.path;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.codehaus.plexus.component.annotations.Component;

/**
 * Normalizes a URL.
 * 
 * @author Benjamin Bentmann
 */
@Component( role = UrlNormalizer.class )
public class DefaultUrlNormalizer
    implements UrlNormalizer
{

    public String normalize( String url )
    {
        String result = url;

        if ( result != null )
        {
            result = result.replaceAll( "(?<![:/])/+", "/" );

            while ( true )
            {
                int idx = result.indexOf( "/../" );
                if ( idx <= 0 )
                {
                    break;
                }
                int parent = result.lastIndexOf( '/', idx - 1 );
                if ( parent < 0 )
                {
                    break;
                }
                result = result.substring( 0, parent ) + result.substring( idx + 3 );
            }
        }

        return result;
    }

}

==> maven-model-builder-3.2.5-sources.jar: org/apache/maven/model/path/DefaultUrlNormalizer.java <==
package org.apache.maven.model.path;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.codehaus.plexus.component.annotations.Component;

/**
 * Normalizes a URL.
 *
 * @author Benjamin Bentmann
 */
@Component( role = UrlNormalizer.class )
public class DefaultUrlNormalizer
    implements UrlNormalizer
{

    public String normalize( String url )
    {
        String result = url;

        if ( result != null )
        {
            while ( true )
            {
                int idx = result.indexOf( "/../" );
                if ( idx <= 0 )
                {
                    break;
                }
                int parent = idx - 1;
                while ( parent >= 0 && result.charAt( parent ) == '/' )
                {
                    parent--;
                }
                parent = result.lastIndexOf( '/', parent );
                if ( parent < 0 )
                {
                    break;
                }
                result = result.substring( 0, parent ) + result.substring( idx + 3 );
            }
        }

        return result;
    }

}
